//============================================================================
// Name        : testbed.cpp
// Author      : 
// Version     :
// Copyright   : Your copyright notice
// Description : Hello World in C++, Ansi-style
//============================================================================

//CGAL
#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Delaunay_triangulation_2.h>
#include <CGAL/Interpolation_traits_2.h>
#include <CGAL/natural_neighbor_coordinates_2.h>
#include <CGAL/interpolation_functions.h>

#include <iostream>

#include "graph.h"

//Functor class for accessing the function values/gradients
	template< class PointWithInfoMap, typename ValueType >
	struct Data_access : public std::unary_function< typename PointWithInfoMap::key_type,
				 std::pair<ValueType, bool> >
	{

	  Data_access(const PointWithInfoMap& points)
			  : _points(points){};

	  std::pair< ValueType, bool>
	  operator()(const typename PointWithInfoMap::key_type& p) const {
		typename PointWithInfoMap::const_iterator mit = _points.find(p);
		if(mit!= _points.end())
		  return std::make_pair(mit->second, true);
		return std::make_pair(ValueType(), false);
	  };

	  const PointWithInfoMap& _points;
	};

typedef CGAL::Exact_predicates_inexact_constructions_kernel    K;
typedef CGAL::Delaunay_triangulation_2<K>                      Delaunay;
typedef CGAL::Interpolation_traits_2<K>                        Traits;
typedef K::FT                                                  Coord_type;
typedef K::Point_2                                             Point;
typedef std::map<Point, Coord_type, K::Less_xy_2>              PointWithInfoMap;
typedef Data_access< PointWithInfoMap, Coord_type >            Value_access;

int main() {

	auto graph = getGraph();
	Delaunay t;
	PointWithInfoMap map;

	for(const auto & p : graph){
		Point x(p.x, p.y);
		t.insert(x);
		map.insert(std::make_pair(x, p.z));
	}

	printf("Triangulation is %s\n", (t.is_valid() ? "valid" : "incorrect"));

	double xx = 122.5;
	double yy = 0.031250;

	Point p(xx, yy);

	printf("begin search\n");
	std::vector<std::pair<Point, Coord_type> > coords;
	auto nn = CGAL::natural_neighbor_coordinates_2(t, p,
			std::back_inserter(coords));

	if(!nn.third){ //neighbor finding was NOT successfull, return standard value
		printf("ERROR\n");
		return 4;
	}

	printf("found neighbors %lu\n", coords.size());

	Coord_type res = CGAL::linear_interpolation(coords.begin(), coords.end(),
			nn.second, Value_access(map));

	printf("%f\n", res);

}
